@@ -10,7 +10,7 @@ NULL
 
 #' The ncyc_map data
 #'
-#' The ncyc_map data is used for the parsing the 'Ncyc' metagenomic results and add the N cycle pathway information to the 'tax_table' of 'microtable' object.
+#' The ncyc_map data is used for the parsing the 'NCycDB' metagenomic results and add the N cycle pathway information to the 'tax_table' of 'microtable' object.
 #'
 #' @docType data
 #' @keywords data.frame
@@ -18,6 +18,16 @@ NULL
 #' @usage data(ncyc_map)
 NULL
 
+#' The pcyc_map data
+#'
+#' The pcyc_map data is used for the parsing the 'PCycDB' metagenomic results and add the P cycle pathway information to the 'tax_table' of 'microtable' object.
+#'
+#' @docType data
+#' @keywords data.frame
+#' @name pcyc_map
+#' @usage data(pcyc_map)
+NULL
+
 #' The MetaCyc_pathway_map data
 #'
 #' The MetaCyc_pathway_map data is a manually curated 'MetaCyc' pathway hierarchical structure data. 
